package CUPS::Query;

use 5.008008;
use strict;
use warnings;

require Exporter;

our @ISA = qw(Exporter);

# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.

# This allows declaration	use CUPS::Query ':all';
# If you do not need this, moving things directly into @EXPORT or @EXPORT_OK
# will save memory.
our %EXPORT_TAGS = ( 'all' => [ qw(
	get_default_dest get_resident_fonts get_ttrasterizer get_modelname
) ] );

our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );

our @EXPORT = qw(
	
);

# $Format: "our $VERSION = '$ProjectVersion$';"$
our $VERSION = '0.13';

require XSLoader;
XSLoader::load('CUPS::Query', $VERSION);

# Preloaded methods go here.

1;
__END__
# Below is the documentation for the module.

=head1 NAME

CUPS::Query - Interface to CUPS used by includefonts

=head1 SYNOPSIS

  use CUPS::Query ':all';
  $printer = get_default_dest();
  @resident_fonts = @{get_resident_fonts($printer)}
  $rasterizer = get_ttrasterizer($printer)

=head1 DESCRIPTION

This module contains Perl interfaces to CUPS print system functions
required by the includefonts(1) utility.

This module is only designed for includefonts(1) and is not intended
to be used by other programs.

The following functions are available:

=over 4

=item B<get_default_dest()>

Get the name of the default printer or class for the default server.
This function returns the name of the default printer or class as
defined by the LPDEST or PRINTER environment variables.  If these
environment variables are not set, the server default destination is
returned.

=item B<get_resident_fonts(I<destination>)>

Get the resident fonts for the printer I<destination>, as specified in
the PPD file for that printer.  This function returns a reference to
an array of font names.

=item B<get_ttrasterizer(I<destination>)>

Returns the string "Type42" if the printer I<destination> contains a
TrueType rasterizer.

=item B<get_modelname(I<destination>)>

Returns the model name of the printer I<destination> as specified in
the PPD file.

=back

=head1 VERSION

$ProjectVersion: 0.13 $

=head1 AUTHOR

Michael Piotrowski, E<lt>mxp@dynalabs.deE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright (C) 2006 by Michael Piotrowski

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.8.8 or,
at your option, any later version of Perl 5 you may have available.

=head1 SEE ALSO

L<includefonts(1)>

=cut
