#include "EXTERN.h"
#include "perl.h"
#include "XSUB.h"

#include "ppport.h"

#include <unistd.h>
#include <cups/cups.h>
#include <cups/ppd.h>

AV *get_resident_fonts (const char *dest)
{
   char *ppd_filename;
   ppd_file_t *ppd;
   ppd_attr_t *attr;
   AV *perl_array;

   perl_array = newAV();
   ppd_filename = (char *) cupsGetPPD(dest);
   ppd = ppdOpenFile(ppd_filename);

   attr = ppdFindAttr(ppd, "Font", NULL);

   if (attr)
      av_push(perl_array, newSVpv(attr->spec, 0));

   while (attr = ppdFindNextAttr(ppd, "Font", NULL))
   {
      av_push(perl_array, newSVpv(attr->spec, 0));
   }

   unlink(ppd_filename);
   return perl_array;
}

SV *get_attr_value (const char *dest, const char *keyword)
{
   char *ppd_filename;
   ppd_file_t *ppd;
   ppd_attr_t *attr;

   ppd_filename = (char *) cupsGetPPD(dest);
   ppd = ppdOpenFile(ppd_filename);

   attr = ppdFindAttr(ppd, keyword, NULL);

   unlink(ppd_filename);

   if (attr == NULL)
      return &PL_sv_undef;

   return newSVpv(attr->value, 0);
}

MODULE = CUPS::Query		PACKAGE = CUPS::Query		

PROTOTYPES: ENABLE

char *
get_default_dest()
   CODE:
     RETVAL = (char *) cupsGetDefault();
   OUTPUT:
     RETVAL

AV *
get_resident_fonts(dest)
   const char *dest
   CODE:
     RETVAL = get_resident_fonts(dest);
   OUTPUT:
     RETVAL

SV *
get_ttrasterizer(dest)
   const char *dest
   CODE:
     RETVAL = get_attr_value(dest, "TTRasterizer");
   OUTPUT:
     RETVAL

SV *
get_modelname(dest)
   const char *dest
   CODE:
     RETVAL = get_attr_value(dest, "ModelName");
   OUTPUT:
     RETVAL
