/*
 * pslines.c	break up a PostScript job file into lines
 * ------------------------------------------------------
 * 1.00 - 1996-01-14/Bl
 *	1st hack
 * 1.01 - 1996-01-15/Bl
 *	A/UX compatibility
 * 1.02 - 1996-01-15/Bl
 *	Break lines at white space positions
 *	Added option -only to pass DSC comments only
 *
 * Implementation remarks:
 *	As this program evaluates UNIX, Macintosh, and DOS end of line conventions,
 *	it is not suitable to port this program to BS2000.
 */
static char what[] = "pslines.c	01.01	1996-01-14 SRZB/Bl";

#include <stdio.h>
#include <errno.h>
#include <string.h>
#ifdef AUX
#include <local/strerror.h>
#endif

static int retc;

/*
 * Write error message top stderr
 */

void errmsg (errnum, msg)
	int errnum;
	char *msg;
{
	fprintf (stderr, "ERROR: %s", msg);
	if (errnum)
		fprintf (stderr, ": errno = %d (%s)\n", errnum, strerror (errnum));
	else
		fprintf (stderr, "\n");
}

	

/*
 * Get a line from input file up to and including end of line
 * End of line may be either 0x0a, 0x0d or 0x0d 0x0a 
 * (i. e. UNIX, Macintosh, or DOS convention)
 */

int getline (line, maxl, fdi, restp, restl)
	char *line;
	int maxl;
	FILE *fdi;
	char **restp;		/* start of word rest */
	int *restl;		/* length of word rest */
{
	char *lp, *rp;
	int eol;
	int ci, c2, i;
	
	lp = line;
	eol = 0;
	maxl--;
	/* Handle word rest of last buffer */
	if (*restl)
	{
		rp = *restp;
		for (i = 0; i < *restl; i++) *lp++ = *rp++;
		maxl -= *restl;
	}
	*restp = NULL; *restl = 0;
	while (!eol)
	{
		ci = getc (fdi);
		if (ferror (fdi))
		{
			if ((errno == EAGAIN) || (errno == EINTR))
				continue;
			else
			{
				errmsg (errno, "getline");
				return (-1);
			}
		}
		switch (ci)
		{
			case 0x0a:
				*lp = 0x00;
				eol = 1;
				break;
			case 0x0d:
				*lp = 0x00;
				eol = 1;
				c2 = getc (fdi);
				if (ferror (fdi))
				{
					if ((errno == EAGAIN) || (errno == EINTR))
						continue;
					else
					{
						errmsg (errno, "getline");
						return (-1);
					}
				}
				if (c2 != 0x0a) ungetc (c2, fdi);
				break;
			case EOF:
				*lp = 0x00;
				eol = 1;
				break;
			case ' ':
			case 0x09:
				*restp = lp;
			default:
				*lp++ = ci;
				if (--maxl == 0)
				{
					if (*restp != NULL)
					{
						**restp = 0x00; (*restp)++;
						*restl = (int)(lp - *restp);
						lp = *restp - 1; 	/* for return value */
					}
					else *lp = 0x00;
					eol = 1;
				}
		}
	}
	return ((int)(lp - line));	/* return length of line excluding eol char */
}


/*
 * Check for embedded data comment
 */

int bulk (line)
	char *line;
{
	char *keywd, *alnum, *what, *bytesorlines;
	int count, tbulk;

	if (strncmp (line, "%%", 2) != 0)
		return (0);	/* is not a DSC comment, do nothing */

	keywd = strtok (line, " \t");
	if (strncmp (keywd, "%%BeginBinary", 13) == 0)
	{
		tbulk = 1;
	}
	else if (strncmp (keywd, "%%BeginData", 11) == 0)
	{
		tbulk = 0;
	}
	else
		return (0);	/* not a bulk data comment */

	if ((alnum = strtok (NULL, " \t")) == NULL)
		return (0);
	count = atoi (alnum);
	if (tbulk)
		return (count);	/* old form */
	if ((what = strtok (NULL, " \t")) == NULL)
		return (0);	/* default Binary Bytes */
	if (strcmp (what, "Binary") == 0) tbulk = 1;
	else if (strcmp (what, "Hex") == 0) tbulk = 2;
	else if (strcmp (what, "ASCII") == 0) tbulk = 3;
	else tbulk = 4;		/* ill formed comment */
	if ((bytesorlines = strtok (NULL, " \t")) == NULL)
		return (count);	/* default Bytes */
	if (strcmp (bytesorlines, "Lines") == 0)
	{
		if ((tbulk == 1) || (tbulk == 4))
		{
			errmsg (0, "Illegal bulk data specification");
			retc = 1;	/* warning */
		}
		return (-count);	/* count lines but bytes */
	}
	return (count);
}



/*
 * Skip bulk bytes
 * ---------------
 */
int skipbytes (count, fdi)
	int count;
	FILE *fdi;
{
	return (fseek (fdi, count, SEEK_CUR));
}



/*
 * Copy bulk data
 * --------------
 */
int copybytes (count, fdi)
	int count;
	FILE *fdi;
{
	int bytes, rbytes, wbytes;
	unsigned char buffer[1024];

	while (count)
	{
		bytes = (count <= sizeof (buffer)) ? count : sizeof (buffer);
		rbytes = fread (buffer, 1, bytes, fdi);
		if (ferror (fdi))
		{
			errmsg (errno, "copybytes read");
			return (-1);
		}
		count -= rbytes;
		while (rbytes)
		{
			wbytes = fwrite (buffer, 1, rbytes, stdout);
			if (ferror (stdout))
			{
				errmsg (errno, "copybytes write");
				return (-1);
			}
			rbytes -= wbytes;
		}
	}
	return (0);
}



/*
 * Write a line to stdout
 * ----------------------
 */
int putline (line, eol)
	char *line;
	int eol;
{
	int rc;

	if (eol)
		rc = puts (line);
	else
		rc = fputs (line, stdout);
	if (rc < 0)
	{
		errmsg (errno, "putline");
	}
	return (rc);
}


/*
 * Usage hints
 * -----------
 */
void usage ()
{
	fprintf (stderr, "Usage: pslines [-skip | -only] file\n\t%s\n\t%s\n\t%s\n",
			"-skip\tdo not pass embedded bulk data",
			"-only\t to pass only DSC comments",
			"file\tinput file to be processed");
	exit (255);
}



/*
 * Main program
 * ------------
 */
int main (argc, argv)
	int argc;
	char **argv;
{
	int skip, only;
	FILE *fdi;
	int errnum, argn, ll;
	int cnt;
	char temp[256];
	char linbuf[512];
	char *prest;
	int lrest, rc;

	if (argc < 2)
	{
		usage ();
	}
	if (*argv[1] == '-')
	{
		if (strcmp (argv[1], "-skip") == 0)
		{
			skip = 1;
			only = 0;
			argn = 2;
		}
		else if (strcmp (argv[1], "-only") == 0)
		{
			skip = 1;
			only = 1;
			argn = 2;
		}
		else
		{
			usage ();
		}
	}
	else 
	{
		skip = 0;
		only = 0;
		argn = 1;
	}

	if ((fdi = fopen (argv[argn], "r")) == NULL)
	{
		errnum = errno;
		(void)sprintf (temp, "open \"%s\"", argv[1]);
		errmsg (errnum, temp);
		exit (2);
	}
	prest = NULL; lrest = 0;
	while (!feof (fdi))
	{
		if ((ll = getline (linbuf, sizeof (linbuf), fdi, &prest, &lrest)) < 0)
			exit (3);
		if ((feof (fdi)) && (ll == 0))
			break;
		if (only)
		{
			if ((linbuf[0] == '%') && (linbuf[1] == '%'))
				rc = putline (linbuf, 1);
			else rc = 0;
		}
		else rc = putline (linbuf, 1);
		if (rc < 0) exit (4);
		if ((cnt = bulk (linbuf)) != 0)
		{
			if (cnt > 0)
			{
				if (skip) 
				{
					if (skipbytes (cnt, fdi) < 0) exit (3);
				}
				else 
				{
					if (copybytes (cnt, fdi) < 0) exit (3);
				}
			}
			else
			{
				while (cnt++)
				{
					if (getline (linbuf, sizeof (linbuf), fdi, &prest, &lrest) < 0) 
						exit (3);
					if (skip) continue;
					if (putline (linbuf, 0) < 0) exit (4);
				}
			}
		}
	}
	exit (retc);			/* in case of warnings */
}
